<?php

namespace Imoje\Payment;

/**
 * Class CartData
 *
 * @package Imoje\Payment
 */
class Invoice
{

    const TAX_23 = 'TAX_23';
    const TAX_22 = 'TAX_22';
    const TAX_8 = 'TAX_8';
    const TAX_7 = 'TAX_7';
    const TAX_5 = 'TAX_5';
    const TAX_3 = 'TAX_3';
    const TAX_0 = 'TAX_0';
    const TAX_EXEMPT = 'TAX_EXEMPT';
    const TAX_NOT_LIABLE = 'TAX_NOT_LIABLE';
    const TAX_REVERSE_CHARGE = 'TAX_REVERSE_CHARGE';
    const TAX_NOT_EXLUCDING = 'TAX_NOT_EXCLUDING';

    const SHOP_TAX_EXEMPT = 'zw';

    const BUYER_PERSON = 'PERSON';
    const BUYER_COMPANY = 'COMPANY';

    const VAT_ID = 'VAT_ID';

    const VAT_COUNTRY_ALPHA2 = 'PL';
    /**
     * @var array
     */
    protected $buyer;

    /**
     * @var array
     */
    protected $positions;

    /**
     * @param string $name
     * @param int $code
     * @param int $quantity
     * @param string $taxStake
     * @param string $grossAmount
     * @param int $discountAmount
     *
     * @return void
     */
    public function addItem($name, $code, $quantity, $taxStake, $grossAmount, $discountAmount = 0)
    {

        $position = [
            'name' => $name,
            'code' => (string)$code,
            'quantity' => (float)$quantity,
            'unit' => 'Sztuki',
            'taxStake' => $taxStake,
            'grossAmount' => (int)$grossAmount,
        ];

        if ($discountAmount) {
            $position['discountAmount'] = (int)$discountAmount;
        }

        $this->positions[] = $position;
    }

    /**
     * @param string $type
     * @param string $email
     * @param string $fullName
     * @param string $street
     * @param string $city
     * @param string $postalCode
     * @param string $countryCodeAlpha2
     * @param string $idCountryCodeAlpha2
     * @param string $idType
     * @param string $idNumber
     *
     * @return Invoice
     */
    public function setBuyer($type, $email, $fullName, $street, $city, $postalCode, $countryCodeAlpha2, $idCountryCodeAlpha2 = '', $idType = '', $idNumber = '')
    {

        $array = [
            'type' => $type,
            'email' => $email,
            'fullName' => $fullName,
            'street' => $street,
            'city' => $city,
            'postalCode' => $postalCode,
            'countryCodeAlpha2' => $countryCodeAlpha2,
        ];

        $this->buyer = $array;

        return $this;
    }

    /**
     * @param string $idCountryCodeAlpha2
     * @param string $idNumber
     *
     * @return void
     */
    public function setCompanyBuyer($idCountryCodeAlpha2, $idNumber)
    {
        $this->buyer['type'] = self::BUYER_COMPANY;
        $this->buyer['idCountryCodeAlpha2'] = $idCountryCodeAlpha2;
        $this->buyer['idType'] = self::VAT_ID;
        $this->buyer['idNumber'] = $idNumber;
    }

    /**
     * @return array|string
     */
    public function prepare($isApi)
    {

        $array = [
            'buyer' => $this->buyer,
            'positions' => $this->positions,
        ];

        return $isApi
            ? $array
            : base64_encode(gzencode(json_encode($array), 5));
    }

    /**
     * @param CartData $cart
     * @param string $email
     * @param bool $isApi
     *
     * @return array|string
     */
    public static function get($cart, $email, $isApi)
    {

        if (!($cart instanceof CartData)) {
            return [];
        }

        $cart = $cart->prepareCartDataArray();

        $invoice = new Invoice();

        if (isset($cart['items']) && $cart['items']) {

            foreach ($cart['items'] as $item) {

                $tax = null;
                try {
                    $tax = constant('self::TAX_' . $item['vat']);
                } catch (\Error $e) {
                }

                if (!$tax) {
                    return [];
                }

                $invoice->addItem($item['name'],
                    $item['id'],
                    $item['quantity'],
                    $tax,
                    $item['amount']
                );
            }

            if (isset($cart['shipping']) && $cart['shipping']) {
                $invoice->addItem($cart['shipping']['name'],
                    1,
                    1,
                    constant('\Imoje\Payment\Invoice::TAX_' . $cart['shipping']['vat']),
                    $cart['shipping']['amount']
                );
            }

            $type = Invoice::BUYER_PERSON;

            $vatNumber = (isset($cart['address']['billing']['vatNumber']) && $cart['address']['billing']['vatNumber'])
                ? $cart['address']['billing']['vatNumber']
                : ((isset($cart['address']['delivery']['vatNumber']) && $cart['address']['delivery']['vatNumber'])
                    ? $cart['address']['delivery']['vatNumber']
                    : '');

            $invoice->setBuyer($type,
                $email,
                $cart['address']['billing']['name'],
                $cart['address']['billing']['street'],
                $cart['address']['billing']['city'],
                $cart['address']['billing']['postalCode'],
                $cart['address']['billing']['country']
            );

            if ($vatNumber) {
                $vatNumber = trim($vatNumber);

                $vatCountryAlpha2 = self::VAT_COUNTRY_ALPHA2;

                if (strlen($vatNumber) > 2) {
                    $firstTwo = substr($vatNumber, 0, 2);

                    if (ctype_alpha($firstTwo)) {
                        $vatCountryAlpha2 = $firstTwo;
                    }
                }

                $invoice->setCompanyBuyer($vatCountryAlpha2, $vatNumber);
            }

            return $invoice->prepare($isApi);
        }

        return [];
    }
}
